@@ -658,22 +658,20 @@ $(document).ready(function(){
 """
         if self._add_storage:
             scriptmiddle += self._add_storage
-        scriptend = self._add_jscript
+
+        # and create the script end
+        scriptend = self._add_jscript + """
+  if(SKIPOLE.interval && SKIPOLE.IntervalTarget) {
+    SKIPOLE.interval_id = setInterval(SKIPOLE.refreshjson, SKIPOLE.interval*1000, SKIPOLE.IntervalTarget);
+    }
+"""
+        # SKIPOLE.interval_id is needed so clearInterval(SKIPOLE.interval_id) can be called
+        # if the interval changes
         if self.last_scroll:
             # restore to last store position
             scriptend += """
   SKIPOLE.restorepagepos();
 """
-        # and if a regular refresh is required
-        if self.interval:
-            interval = self.interval * 1000
-            intervalurl = ''
-            intervalurl = skiboot.get_url(self.interval_target, self.proj_ident)
-            if intervalurl:
-                intervalurl = quote(intervalurl, safe='/:?&=')
-                scriptend += """
-  setInterval( SKIPOLE.refreshjson, %s, "%s");
-""" % (interval, intervalurl)
         # and set an event to store window position on unload
         scriptend += """
   if(typeof(Storage) !== "undefined") {
@@ -814,9 +812,30 @@ $(document).ready(function(){
 
     def set_values(self, page_data):
         """Checks for special page template values, then passes on to parent set_values to set the widgets"""
+
+        if 'set_cookie' in page_data:
+            # sets the cookies in the page headers
+            sendcookies = page_data['set_cookie']
+            if sendcookies:
+                for morsel in sendcookies.values():
+                    self.headers.append(("Set-Cookie", morsel.OutputString()))
+            del page_data['set_cookie']                      # delete to stop parent object from testing this item is a widgfield
         if 'CatchToHTML' in page_data:
             self.catch_to_html = page_data['CatchToHTML']
-            del page_data['CatchToHTML']                      # delete to stop parent object from testing this item is a widgfiels
+            del page_data['CatchToHTML']                      # delete to stop parent object from testing this item is a widgfield
+        if 'interval' in page_data:
+            interval = 0
+            try:
+                interval = page_data['interval']
+                del page_data['interval']
+                interval = int(interval)
+            except:
+                pass
+            else:
+                self.interval=interval
+        if 'IntervalTarget' in page_data:
+            self.interval_target = page_data['IntervalTarget']
+            del page_data['IntervalTarget']
         if 'last_scroll' in page_data:
             self.last_scroll = bool(page_data['last_scroll'])
             del page_data['last_scroll']
@@ -868,8 +887,12 @@ $(document).ready(function(){
 SKIPOLE.identdata = '%s';
 SKIPOLE.default_error_widget = '%s';
 SKIPOLE.widget_register = {};
-"""  % (self.ident.proj, self.ident.num, self.ident_data_string, self.default_error_widget)
-
+SKIPOLE.interval = %s;
+"""  % (self.ident.proj, self.ident.num, self.ident_data_string, self.default_error_widget, self.interval)
+        if self.interval_target:
+            intervalurl = skiboot.get_url(self.interval_target, self.proj_ident)
+            if intervalurl:
+                page_javascript += "SKIPOLE.IntervalTarget = \'" + quote(intervalurl, safe='/:?&=') + "\';\n"
         if self.catch_to_html:
             catchurl = skiboot.get_url(self.catch_to_html, self.proj_ident)
             if catchurl:
@@ -928,23 +951,21 @@ class RespondPage(ParentPage):
         # - or more usually, an instance of a child of the Respond class
         self.responder = responder
 
-    def call_responder(self, skicall, form_data, caller_ident, ident_list, rawformdata, error_dict=None):
+    def call_responder(self, skicall, form_data, caller_ident, ident_list, rawformdata):
         """Checks for circulating calls, then updates ident_list with this pages ident,
            then calls the respond objects call method, note rawformdata is a cgi.FieldStorage object"""
         if self.responder is None:
             raise ServerError(message = "No responder has been assigned to this page")
         if self.ident in ident_list:
-            raise ServerError(message="Circulating call to Respond page: ident %s in %s" % (self.ident, ident_list))
+            raise ServerError(message="Circulating call to Respond page: ident %s" % (self.ident,))
         ident_list.append(self.ident)
         proj = self.ident.proj
 
         # create submit_dict
-        if error_dict:
-            skicall.submit_dict = {"responder_brief":self.brief, "number":self.ident.num, 'error_dict':error_dict}
-        else:
-            skicall.submit_dict = {"responder_brief":self.brief, "number":self.ident.num, 'error_dict':{}}
+        skicall.submit_dict["responder_brief"] = self.brief
+        skicall.submit_dict["number"] = self.ident.num
 
-        page = self.responder(skicall, form_data, caller_ident, ident_list, proj, rawformdata, error_dict)
+        page = self.responder(skicall, form_data, caller_ident, ident_list, proj, rawformdata)
         return page
 
     def set_values(self, page_data):
@@ -1301,6 +1322,9 @@ class CSS(ParentPage):
         if 'enable_cache' in page_data:
             self.enable_cache = bool(page_data['enable_cache'])
             del page_data['enable_cache']
+        if 'colour_substitution' in page_data:
+            self.colour_substitution = page_data['colour_substitution']
+            del page_data['colour_substitution']
         if '@import' in page_data:
             if isinstance(page_data['@import'], list):
                 self.imports = page_data['@import']
@@ -1462,6 +1486,21 @@ class JSON(ParentPage):
         if 'throw' in page_data:
             self.content["throw"] = page_data['throw']
             del page_data['throw']
+        if "IntervalTarget" in page_data:
+            url = skiboot.get_url(page_data["IntervalTarget"], proj_ident=self.proj_ident)
+            if url:
+                self.content["IntervalTarget"] = url
+            del page_data["IntervalTarget"]
+        if 'interval' in page_data:
+            interval=0
+            try:
+                interval = page_data['interval']
+                del page_data['interval']
+                interval = int(interval)
+            except:
+                pass
+            else:
+                self.content["interval"]=interval
         if "CatchToHTML" in page_data:
             url = skiboot.get_url(page_data["CatchToHTML"], proj_ident=self.proj_ident)
             if url:
